@@ -1,6 +1,6 @@
 /*
-htop
-(C) 2004 Hisham H. Muhammad
+htop - ProcessList.c
+(C) 2004,2005 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -77,11 +77,6 @@ typedef struct ProcessList_ {
    long int usedSwap;
    long int freeSwap;
 
-   int kernelMajor;
-   int kernelMiddle;
-   int kernelMinor;
-   int kernelTiny;
-
    ProcessField* fields;
    ProcessField sortKey;
    int direction;
@@ -96,36 +91,19 @@ typedef struct ProcessList_ {
 } ProcessList;
 }*/
 
-/* private */
-void ProcessList_getKernelVersion(ProcessList* this) {
-   struct utsname uts;
-   (void) uname(&uts);
-   char** items = String_split(uts.release, '.');
-   this->kernelMajor = atoi(items[0]);
-   this->kernelMiddle = atoi(items[1]);
-   this->kernelMinor = atoi(items[2]);
-   this->kernelTiny = items[3] ? atoi(items[3]) : 0;
-   for (int i = 0; items[i] != NULL; i++) free(items[i]);
-   free(items);
-}
-
 /* private property */
-ProcessField defaultHeaders[LAST_PROCESSFIELD] = { PID, USER, PRIORITY, NICE, M_SIZE, M_RESIDENT, M_SHARE, STATE, PERCENT_CPU, PERCENT_MEM, TIME, COMM, LAST_PROCESSFIELD };
+ProcessField defaultHeaders[] = { PID, USER, PRIORITY, NICE, M_SIZE, M_RESIDENT, M_SHARE, STATE, PERCENT_CPU, PERCENT_MEM, TIME, COMM, LAST_PROCESSFIELD, 0 };
 
 ProcessList* ProcessList_new(UsersTable* usersTable) {
    ProcessList* this;
    this = malloc(sizeof(ProcessList));
    this->processes = TypedVector_new(PROCESS_CLASS, true, DEFAULT_SIZE);
    this->processTable = Hashtable_new(20, false);
-   TypedVector_setCompareFunction(this->processes, Process_compare);
    this->prototype = Process_new(this);
    this->usersTable = usersTable;
    
    /* tree-view auxiliary buffers */
    this->processes2 = TypedVector_new(PROCESS_CLASS, true, DEFAULT_SIZE);
-   TypedVector_setCompareFunction(this->processes2, Process_compare);
-
-   ProcessList_getKernelVersion(this);
 
    FILE* status = fopen(PROCSTATFILE, "r");
    assert(status != NULL);
@@ -152,10 +130,10 @@ ProcessList* ProcessList_new(UsersTable* usersTable) {
       this->totalPeriod[i] = 1;
    }
 
-   this->fields = malloc(sizeof(ProcessField) * LAST_PROCESSFIELD);
+   this->fields = calloc(sizeof(ProcessField), LAST_PROCESSFIELD+1);
    // TODO: turn 'fields' into a TypedVector,
    // (and ProcessFields into proper objects).
-   for (int i = 0; i < LAST_PROCESSFIELD; i++) {
+   for (int i = 0; defaultHeaders[i]; i++) {
       this->fields[i] = defaultHeaders[i];
    }
    this->sortKey = PERCENT_CPU;
@@ -199,28 +177,10 @@ void ProcessList_invertSortOrder(ProcessList* this) {
       this->direction = 1;
 }
 
-void ProcessList_sortKey(ProcessList* this, int delta) {
-   assert(delta == 1 || delta == -1);
-   int i = 0;
-   while (this->fields[i] != this->sortKey)
-      i++;
-   i += delta;
-   if (i < 0) {
-      i = 0;
-      while (this->fields[i] != LAST_PROCESSFIELD)
-         i++;
-      i--;
-   } else if (this->fields[i] == LAST_PROCESSFIELD)
-      i = 0;
-   this->sortKey = this->fields[i];
-   this->direction = 1;
-   // Weird code...
-}
-
 RichString ProcessList_printHeader(ProcessList* this) {
    RichString out = RichString_new();
    ProcessField* fields = this->fields;
-   for (int i = 0; fields[i] != LAST_PROCESSFIELD; i++) {
+   for (int i = 0; fields[i]; i++) {
       char* field = Process_printField(fields[i]);
       if (this->sortKey == fields[i])
          RichString_append(&out, CRT_colors[PANEL_HIGHLIGHT_FOCUS], field);
@@ -242,8 +202,11 @@ void ProcessList_add(ProcessList* this, Process* p) {
 
 void ProcessList_remove(ProcessList* this, Process* p) {
    Hashtable_remove(this->processTable, p->pid);
+   ProcessField pf = this->sortKey;
+   this->sortKey = PID;
    int index = TypedVector_indexOf(this->processes, p);
    TypedVector_remove(this->processes, index);
+   this->sortKey = pf;
 }
 
 Process* ProcessList_get(ProcessList* this, int index) {
@@ -467,18 +430,15 @@ void ProcessList_processEntries(ProcessList* this, char* dirname, int parent, fl
                goto errorReadingProcess;
             }
 
-            char* cmdline = process->comm;
-            int amtRead = fread(cmdline, 1, PROCESS_COMM_LEN - 1, status);
+            int amtRead = fread(command, 1, PROCESS_COMM_LEN - 1, status);
             if (amtRead > 0) {
                for (int i = 0; i < amtRead; i++)
-                  if (cmdline[i] == '\0' || cmdline[i] == '\n')
-                     cmdline[i] = ' ';
-               cmdline[amtRead] = '\0';
-            }
-            else {
-               strncpy(process->comm, command, PROCESS_COMM_LEN);
-               cmdline[PROCESS_COMM_LEN] = '\0';
+                  if (command[i] == '\0' || command[i] == '\n')
+                     command[i] = ' ';
+               command[amtRead] = '\0';
             }
+            command[PROCESS_COMM_LEN] = '\0';
+            process->comm = String_copy(command);
             fclose(status);
          }
 
@@ -561,26 +521,24 @@ void ProcessList_scan(ProcessList* this) {
    status = fopen(PROCSTATFILE, "r");
    assert(status != NULL);
    for (int i = 0; i <= this->processorCount; i++) {
+      char buffer[256];
       int cpuid;
-      if (this->kernelMajor == 2 && this->kernelMiddle <= 4) {
-         if (i == 0) {
-            fscanf(status, "cpu  %ld %ld %ld %ld\n", &usertime, &nicetime, &systemtime, &idletime);
-         } else {
-            fscanf(status, "cpu%d %ld %ld %ld %ld\n", &cpuid, &usertime, &nicetime, &systemtime, &idletime);
-            assert(cpuid == i - 1);
-         }
-         totaltime = usertime + nicetime + systemtime + idletime;
-      } else {
-         long int ioWait, irq, softIrq;
-         if (i == 0)
-            fscanf(status, "cpu  %ld %ld %ld %ld %ld %ld %ld\n", &usertime, &nicetime, &systemtime, &idletime, &ioWait, &irq, &softIrq);
-         else {
-            fscanf(status, "cpu%d %ld %ld %ld %ld %ld %ld %ld\n", &cpuid, &usertime, &nicetime, &systemtime, &idletime, &ioWait, &irq, &softIrq);
-            assert(cpuid == i - 1);
-         }
-         systemtime += ioWait + irq + softIrq;
-         totaltime = usertime + nicetime + systemtime + idletime;
+      long int ioWait, irq, softIrq, steal;
+      ioWait = irq = softIrq = steal = 0;
+      // Dependending on your kernel version,
+      // 5, 7 or 8 of these fields will be set.
+      // The rest will remain at zero.
+      fgets(buffer, 255, status);
+      if (i == 0)
+         sscanf(buffer, "cpu  %ld %ld %ld %ld %ld %ld %ld %ld\n", &usertime, &nicetime, &systemtime, &idletime, &ioWait, &irq, &softIrq, &steal);
+      else {
+         sscanf(buffer, "cpu%d %ld %ld %ld %ld %ld %ld %ld %ld\n", &cpuid, &usertime, &nicetime, &systemtime, &idletime, &ioWait, &irq, &softIrq, &steal);
+         assert(cpuid == i - 1);
       }
+      // Fields existing on kernels >= 2.6
+      // (and RHEL's patched kernel 2.4...)
+      systemtime += ioWait + irq + softIrq + steal;
+      totaltime = usertime + nicetime + systemtime + idletime;
       assert (usertime >= this->userTime[i]);
       assert (nicetime >= this->niceTime[i]);
       assert (systemtime >= this->systemTime[i]);
