@@ -1,6 +1,6 @@
 /*
-htop
-(C) 2004 Hisham H. Muhammad
+htop - UsersTable.c
+(C) 2004,2005 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -17,12 +17,6 @@ in the source distribution for its full text.
 #include "debug.h"
 #include <assert.h>
 
-/*{
-#ifndef SYSCONFDIR
-#define SYSCONFDIR "/etc"
-#endif
-}*/
-
 /*{
 typedef struct UsersTable_ {
    Hashtable* users;
@@ -53,6 +47,10 @@ char* UsersTable_getRef(UsersTable* this, int uid) {
    return name;
 }
 
-int UsersTable_size(UsersTable* this) {
+inline int UsersTable_size(UsersTable* this) {
    return (Hashtable_size(this->users));
 }
+
+inline void UsersTable_foreach(UsersTable* this, Hashtable_PairFunction f, void* userData) {
+   Hashtable_foreach(this->users, f, userData);
+}
